@@ -169,6 +169,8 @@ class AtomJsonMap(BinaryMap):
         """
         Convert atom {symbol: fraction} list to numeric features
         """
+        if isinstance(atoms, dict):
+            atoms = [{k: v} for k, v in atoms.items()]
 
         features = []
         for atom in atoms:
@@ -179,14 +181,20 @@ class AtomJsonMap(BinaryMap):
                 else:
                     emb += np.array(self.embedding_dict[k])
             features.append(emb)
-        return np.array(features).reshape((len(atoms), -1))
+        if len(atoms) ==1:
+            return np.array(features).ravel()
+        else:
+            return np.array(features).reshape((len(atoms), -1))
 
     def convert_number(self, atoms: List[int]) -> np.ndarray:
 
         atoms_name = [ALL_N_ELE_MAP[i] for i in atoms]
         features = [self.embedding_dict[i] for i in atoms_name]
 
-        return np.array(features).reshape((len(atoms), -1))
+        if len(atoms) == 1:
+            return np.array(features).ravel()
+        else:
+            return np.array(features).reshape((len(atoms), -1))
 
 
 class AtomTableMap(BinaryMap):
@@ -219,11 +227,17 @@ class AtomTableMap(BinaryMap):
     >>> s = [{"H": 2, }, {"Po": 1}]  #[i.species.as_dict() for i in pymatgen.structure.sites]
     >>> a = tmps.convert(s)
     ...
-    >>> tmps = AtomTableMap(search_tp="name")
+    >>> tmps = AtomTableMap(search_tp="name",tablename="oe.csv")
     >>> s = [[{"H": 2, }, {"Po": 1}],[{"H": 2, }, {"Po": 1}]]
     >>> a = tmps.transform(s)
     ...
 
+    >>> tmps = AtomTableMap(tablename=None) or
+    >>> tmps = AtomTableMap(tablename="element_table.csv")
+    >>> s = [{"H": 2, }, {"Pd": 1}]
+    >>> b = tmps.convert(s)
+    ...
+
     """
 
     def __init__(self, tablename: Union[str, np.ndarray, pd.DataFrame] = "oe.csv",
@@ -284,6 +298,8 @@ class AtomTableMap(BinaryMap):
         """
         Convert atom {symbol: fraction} list to numeric features
         """
+        if isinstance(atoms, dict):
+            atoms = [{k: v} for k, v in atoms.items()]
 
         features = []
         for atom in atoms:
@@ -294,7 +310,10 @@ class AtomTableMap(BinaryMap):
                 else:
                     emb += self.da.loc[k, :].values
             features.append(emb)
-        return np.array(features).reshape((len(atoms), -1))
+        if len(atoms) == 1:
+            return np.array(features).ravel()
+        else:
+            return np.array(features).reshape((len(atoms), -1))
 
     def convert_number(self, atoms: List[int]) -> np.ndarray:
         """
@@ -497,6 +516,8 @@ class AtomPymatgenPropMap(BinaryMap):
         """
         Convert atom {symbol: fraction} list to numeric features
         """
+        if isinstance(atoms, dict):
+            atoms = [{k: v} for k, v in atoms.items()]
 
         features = []
         for atom in atoms:
@@ -524,7 +545,10 @@ class AtomPymatgenPropMap(BinaryMap):
         if len(set([i.shape for i in features])) != 1:
             print("The preprocessing after func are with different size.",
                   "please check you func, which keep the number of results consistent.")
-        return np.array(features).reshape((len(atoms), -1))
+        if len(atoms) == 1:
+            return np.array(features).ravel()
+        else:
+            return np.array(features).reshape((len(atoms), -1))
 
     def convert_number(self, atoms: List[int]) -> np.ndarray:
         """
@@ -541,7 +565,10 @@ class AtomPymatgenPropMap(BinaryMap):
         if len(set([i.shape for i in features])) != 1:
             print("The preprocessing after func are with different size.",
                   "please check you func, which keep the number of results consistent.")
-        return np.array(features).reshape((len(atoms), -1))
+        if len(atoms) == 1:
+            return np.array(features).ravel()
+        else:
+            return np.array(features).reshape((len(atoms), -1))
 
     def __add__(self, other):
         if isinstance(other, AtomPymatgenPropMap):
