@@ -1,8 +1,10 @@
 import unittest
 
-from featurebox.featurizers.mapper import StructurePymatgenPropMap
 import pandas as pd
 
+from featurebox.featurizers.state.state_mapper import StructurePymatgenPropMap
+
+
 class MyTestCase(unittest.TestCase):
     def setUp(self) -> None:
         self.data = pd.read_pickle("data_structure.pkl_pd")
@@ -15,12 +17,12 @@ class MyTestCase(unittest.TestCase):
     #     data = sppm.fit_transform(self.data0_3)
 
     def test_something2(self):
-        sppm = StructurePymatgenPropMap(prop_name=["lengths","angles","get_brillouin_zone()"])
+        sppm = StructurePymatgenPropMap(prop_name=["lengths", "angles", "get_brillouin_zone()"])
         data = sppm.fit_transform([i._lattice for i in self.data0_3])
 
-
     def test_something3(self):
-        sppm = StructurePymatgenPropMap(prop_name=["_lattice.lengths","_lattice.angles","_lattice.get_brillouin_zone()"])
+        sppm = StructurePymatgenPropMap(
+            prop_name=["_lattice.lengths", "_lattice.angles", "_lattice.get_brillouin_zone()"])
         data = sppm.fit_transform(self.data0_3)
         data = sppm.fit_transform(self.data0_3)
 
